# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
from baas.core.plugins import Plugin

class Example(Plugin):

    def get_map(self):
        """
            returns the command map for the plugin
        """
        cmd_map = [('example',self.example_action)]
        return cmd_map

    def get_help(self):
        """
            returns the help text for the plugin
        """
        return {'commands':['example:foo - gives you a bla for a foo'],'additional':[]}

    def example_action(self, term):
        '''
        example plugin
        '''
        result = ''
        term = term.strip()
        if term == '':
            return "Please specify your term"

        if term != 'foo':
            text = "I don't understand...?"
        else:
            text = "bla"

        return self.render(data=text, title=None)

    def render_xmpp(self, data, title):
        #result = title+"\n" #Would be the title
        result = data
        return self.strip_tags(result)

    def render_wave(self, data, title):
        #Would be the title
        result = " <br/><br/>" #<b>%s</b><br/>" % self.xmlify(title)
        result += "<b>%s</b>" % self.xmlify(data)
        return result


# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
from urllib import quote_plus
from baas.core.plugins import Plugin

class Twingly(Plugin):

    def get_map(self):
        """
            returns the command map for the plugin
        """
        cmd_map = [('blog',self.blogs), ('micro',self.microblogs)]
        return cmd_map

    def get_help(self):
        """
            returns the help text for the plugin
        """
        return {'commands':['blog:word - blog search via twingly','micro:word - microblog search via twingly'],'additional':[]}

    def _search(self, term, content):

        term = term.strip()
        if term == '':
            return "Please specify your search term"

        twingly_rss = "http://www.twingly.com/search.rss?q=%s&sort=published&content=%s"  % (quote_plus(term.encode('utf-8')), content)

        feed = self.load_feed(twingly_rss)

        if content == 'blog': limit = 5
        else: limit = 10

        title = '%s search for %s' % (content, term)
        hits = None
        if feed.entries:
            hits = feed.entries[0:limit]

        return self.render(data=hits, title=title, extra_format=content)

        if feed.entries:            
            for row in feed.entries[0:limit]:
                desc = row["summary"]+"\n" if len(row["summary"]) > 0 else ''
                result += '* %s' % row["title"]
                if desc != '' and content== 'blog':
                    result += ': %s' % desc
                result += '%s\n' % row["link"]
            result = self.htmlentities_decode(result)
        else:
            result += 'No sites found!'

        return result

    def blogs(self, term):
        '''
        blog search
        '''
        result = self._search(term, 'blog')
        return result


    def microblogs(self, term):
        '''
        microblog search
        '''
        result = self._search(term, 'microblog')
        return result

    def render_xmpp_blog(self, data, title):
        '''
        renders the blog result for xmpp responses
        '''
        result = title+"\n"
        if data:
            for row in data:
                desc = row["summary"]+"\n" if len(row["summary"]) > 0 else ''
                result += '* %s' % row["title"]
                if desc != '':
                    result += ': %s' % desc
                result += '%s\n' % row["link"]
            result = self.htmlentities_decode(result)
        else:
            result += 'No sites found!'
        return self.strip_tags(result)

    def render_wave_blog(self, data, title):
        '''
        renders the blog result for wave responses
        '''
        result = " <br/><br/><b>%s</b><br/>" % self.xmlify(title)
        if data:
            for row in data:
                result += '<a href="%s">%s</a><br/>' % (self.xmlify(row["link"]), self.xmlify(row["title"]))
                desc = row["summary"] if len(row["summary"]) > 0 else None
                if desc:
                    result += "%s<br/>" % self.xmlify(self.htmlentities_decode(desc))
                result += '<br/>'
        else:
            result += 'No sites found!'
        return result

    def render_xmpp_microblog(self, data, title):
        '''
        renders the microblog result for xmpp responses
        '''
        result = title+"\n"
        if data:
            for row in data:
                desc = row["summary"]+"\n" if len(row["summary"]) > 0 else ''
                result += '* %s' % row["title"]
                if desc != '':
                    result += ': %s' % desc
                result += '%s\n' % row["link"]
            result = self.htmlentities_decode(result)
        else:
            result += 'No sites found!'
        return self.strip_tags(result)

    def render_wave_microblog(self, data, title):
        '''
        renders the microblog result for wave responses
        '''
        result = " <br/><br/><b>%s</b><br/>" % self.xmlify(title)
        if data:
            for row in data:
                result += '%s <a href="%s">link</a><br/><br/>' % (self.xmlify(row["title"]), self.xmlify(row["link"]))
        else:
            result += 'No sites found!'
        return result 



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import ConfigParser
from waveapi import events
from waveapi import model
from waveapi import robot
from waveapi.ops import OpBuilder
from baas.core.plugins import PluginLoader
from baas.core.helpers import *

config = ConfigParser.ConfigParser()
config.read("../../baas.cfg")

pluginHnd = PluginLoader(config=config, format="wave")
pluginHnd.load_plugins()
pluginHnd.load_map()
pluginHnd.load_help()
commands= pluginHnd.commands


def OnRobotAdded(properties, context):
    """Invoked when the robot has been added."""
    reply = "type 'buddy:help' for available commands"
    root_wavelet = context.GetRootWavelet()
    root_wavelet.CreateBlip().GetDocument().SetText(reply)

def OnBlipSubmitted(properties, context):
    """ checks every new blip """
    blip = context.GetBlipById(properties['blipId'])
    text = blip.GetDocument().GetText().strip()

    reply = ''
    if text and text == 'buddy:help':
        help = "\n\n%s" % pluginHnd.help
        blip.GetDocument().SetText("%s\n%s" % (text, help))
    elif text and text.find(':')+1:
        cmd,args=text.split(':',1)
        commando_func = commands.get(cmd)
        if commando_func:
            result_msg = commando_func(args)
            reply = result_msg

    
    if reply != '':
        blip.GetDocument().SetText(" ")
        reply = '<b>asked</b> %s %s' % (xmlify(text), reply)
        builder = OpBuilder(context)
        builder.DocumentAppendMarkup(blip.waveId, blip.waveletId, properties['blipId'], reply)



if __name__ == '__main__':
    myRobot = robot.Robot(config.get('wave','name'),
        image_url=config.get('wave','image_url'),
        version=config.get('wave','version'),
        profile_url=config.get('wave','profile_url'))
    myRobot.RegisterHandler(events.WAVELET_SELF_ADDED, OnRobotAdded)
    myRobot.RegisterHandler(events.BLIP_SUBMITTED, OnBlipSubmitted)
    myRobot.Run()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import sys
import os.path
import traceback
import ConfigParser
from optparse import OptionParser
from twisted.words.protocols.jabber import client, jid
from twisted.words.xish import domish, xmlstream
from twisted.internet import reactor
from baas.core.plugins import PluginLoader

########################### helpers ##############################################
def getTraceback():
    e_info = sys.exc_info()
    e_tb   = "".join(traceback.format_tb(e_info[2]))
    return ' Error Type: '+str(e_info[0])+'\nError Value: '+str(e_info[1])+'\nTraceback:\n'+e_tb+'\n'

class Bot(object):

    def __init__(self, config, ):

        self.config = config
        self.debug = self.config.getint('app','debug')
        self._load_plugins()

        me = jid.JID(self.config.get('bot','jid'))
        self.factory = client.basicClientFactory(me, self.config.get('bot','pwd'))
        self.factory.addBootstrap('//event/stream/authd',self.authd)
        self.factory.addBootstrap(client.BasicAuthenticator.AUTH_FAILED_EVENT, self.authfailedEvent)

        reactor.connectTCP(self.config.get('bot','server'), self.config.getint('bot','port'), self.factory)
        
        reactor.run()
        self.reactor = reactor

    def _load_plugins(self):        
        self.pluginHnd = PluginLoader(config=self.config)
        self.pluginHnd.load_plugins()
        self.pluginHnd.load_map()
        self.pluginHnd.load_help()
        self.commands= self.pluginHnd.commands

    def gotMessage(self, message):
        
        if self.debug:
            print message.toXml()
        text = None
        user = message["from"]
        # sorry for the __str__(), makes unicode happy
        for e in message.elements():
            if e.name == "body":
                text = unicode(e.__str__())
                break

        reply = "type 'help' for available commands"

        try:

            if text and text.find(':')+1:
                cmd,args=text.split(':',1)
                commando_func = self.commands.get(cmd)
                if commando_func:
                    result_msg = commando_func(args)
                    reply = result_msg
                else:
                    reply = 'Uups, commando not known\n'
            elif text and text == 'help':
                reply += "\n\n%s" % self.pluginHnd.help

        except:
            reply = getTraceback()

        response = domish.Element((None, 'message'))
        response['to'] = user
        response["type"] = 'chat'
        response.addElement('body', content=reply)
        
        self.xmlstream.send(response)

    def authfailedEvent(self, xmlstream):
        global reactor
        try:
            self.reactor.stop()
        except:
            pass
        sys.exit('Auth failed!')
        

    def authd(self, xmlstream):
        # need to send presence so clients know we're
        # actually online.
        presence = domish.Element(('jabber:client', 'presence'))
        presence.addElement('status').addContent('Online')

        xmlstream.send(presence)

        # add a callback for the messages
        xmlstream.addObserver('/message', self.gotMessage)
        self.xmlstream = xmlstream

def main():
    usage = "usage: %prog [options] arg"
    parser = OptionParser(usage)
    parser.add_option("-c", "--config", dest="config",
                      help="configuration file (default: /etc/baas.conf)")
    parser.add_option("-s", "--show-conf-example", dest="show_conf_example",
                     action="store_true", default=False, help="show example config file")
    (options, args) = parser.parse_args()

    
    if not options.config:
        config_file = "/etc/baas.conf"
    else:
        config_file = options.config

    if options.show_conf_example:
        f = open(os.path.join(os.path.dirname(os.path.dirname(__file__)), 'conf/baas.conf'))
        conf_example = f.read()
        f.close()
        print conf_example
        sys.exit()
    elif not os.path.isfile(config_file):
        sys.exit('Configuration file does not exist, exiting. Type -h for help.')

    config = ConfigParser.ConfigParser()
    config.read(config_file)

    print "starting buddy as service bot"
    bot= Bot(config)

if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import urllib2
import re
from urllib import quote_plus
from baas.core.plugins import Plugin

try:
    # appengine
    from django.utils import simplejson
except:
    import simplejson

class Translate(Plugin):

    def get_map(self):
        """
            returns the command map for the plugin
        """
        cmd_map = [('tlate',self.translate)]
        return cmd_map

    def get_help(self):
        """
            returns the help text for the plugin
        """
        additional = '''
A translation can have the source (@) and the target (#) language specified by tags
tlate:Wie gehts?  @de #en
tlate:Wie gehts?  @de #es
tlate:How do you do? #de'''

        return {'commands':['tlate:word [@source] [#target] - translates the word from @source in #target, default #en, @ is optional'],'additional':[additional]}

    def _api_request(self, term, target_lang, source_lang=''):
        result = ''
        detected_lang = None
        try:
            url_term = quote_plus(term.encode('utf-8').lower())
            api_url = 'http://www.google.com/uds/Gtranslate?context=22&q=%s&langpair=%s|%s&key=notsupplied&v=1.0' % (url_term, source_lang, target_lang)

            req = urllib2.Request(api_url)
            response = urllib2.urlopen(req).read()
            api_response  = simplejson.loads(response)

            if api_response.get('responseStatus') == 200:
                translate_data = api_response.get('responseData')
                if not source_lang:
                    source_lang = translate_data.get('detectedSourceLanguage')

                result = translate_data.get('translatedText')
        except:
            raise EnvironmentError, 'Translation failed'
        return result, source_lang

    def translate(self, term):
        '''
        example plugin
        '''
        result = ''
        target = 'en'
        source = ''        
        if term == '':
            return "Please specify your text to translate"

        pat = re.compile('(?P<term>[^#@]*)(?P<source>@[^\ #]*)?\ ?(?P<target>#[^\ @]*)?', re.I)
        cmds = pat.search(term)
        if cmds:
            term = cmds.group('term')
            source = cmds.group('source') or ''
            target = cmds.group('target') or target

        target = target.strip('#')
        source = source.strip('@')

        translated_text, detected_lang = self._api_request(term, target, source)
        return self.render(data={'text':translated_text, 'lang': target, 'detected_lang': detected_lang}, title=None)

    def render_xmpp(self, data, title):
        '''
        renders the result for xmpp responses
        '''
        if data.get('text'):
            result = '%s (%s => %s)' % (self.htmlentities_decode(data.get('text')), data.get('detected_lang'), data.get('lang'))
        else:
            result = 'Text translation failed'
        return self.strip_tags(result)

    def render_wave(self, data, title):
        '''
        renders the result for wave responses
        '''
        result = " <br/><br/>"
        if data.get('text'):
            result += '<b>%s</b> <i>(%s =&gt; %s)</i>' \
                % (self.xmlify(self.htmlentities_decode(data.get('text'))),
                    data.get('detected_lang'), data.get('lang'))
        else:
            result += 'Text translation failed'
        return result




# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import urllib2
import urllib
import re
import chardet
from urllib import quote_plus
from xml.dom import minidom
from xml.etree import ElementTree
from baas.core.plugins import Plugin

try:
    # appengine
    from django.utils import simplejson
except:
    import simplejson

class Weather(Plugin):

    def get_map(self):
        """
            returns the command map for the plugin
        """
        cmd_map = [('weather',self.forecast)]
        return cmd_map

    def get_help(self):
        """
            returns the help text for the plugin
        """
        additional = ''''''

        return {'commands':['weather:city [,country] [#lang] - get weather information and forecast'],'additional':[additional]}

    def _fix_invalid_xml(self, xml):
        #0xA0 0x25 0x22 0x2F
        encoding = chardet.detect(xml)['encoding']
        xml = xml.decode(encoding).encode('utf-8')            
        return xml

    def _sanitize(self, string):
        return string.encode('utf-8','ignore')

    def _api_request(self, location, lang):
        """
            makes a api-request and parses result
        """
        result = {'info':{}, 'current':{},'forecast':[]}
        #lang='en'
        try:
            url_term = urllib.urlencode({'weather':location.encode('utf-8').lower(),'hl':lang})
            
            api_url = 'http://www.google.de/ig/api?%s' % (url_term)

            req = urllib2.Request(api_url)
            response = urllib2.urlopen(req).read()
            if response:
                response = self._fix_invalid_xml(response)
                weather = ElementTree.fromstring(response)

                for e in weather.find('weather/forecast_information').getchildren():
                    result['info'][e.tag] = e.attrib.get('data').encode('utf-8','ignore')

                for e in weather.find('weather/current_conditions').getchildren():
                    result['current'][e.tag] = e.attrib.get('data').encode('utf-8','ignore')

                match = 'weather/forecast_conditions'
                forecasts = [e.getchildren() for e in weather.findall(match)]
                for fcast in forecasts:
                    day = {}
                    for e in fcast:
                        day[e.tag] = e.attrib.get('data').encode('utf-8','ignore')
                    result['forecast'].append(day)
        except:
            raise EnvironmentError, 'Forecast failed'
        return result

    def forecast(self, term):
        '''
        requests a weather forecast via google weather api 
        '''
        result = ''
        lang = 'en'
        if term == '':
            return "Please specify your location"

        pat = re.compile('(?P<term>[^#]*)\ ?(?P<lang>#[^\ @]*)?', re.I)
        cmds = pat.search(term)
        if cmds:
            term = cmds.group('term')
            lang = cmds.group('lang') or lang

        lang = lang.strip('#')
        term = term.strip()

        result = self._api_request(term, lang)
        return self.render(data=result, title=None)

    def render_xmpp(self, data, title):
        '''
        renders the result for xmpp responses
        '''
        i = data.get('info')
        result = '%s:\n' % (i.get('city'))#, i.get('current_date_time'))
        c = data.get('current')
        if c.get('condition'): 
            result += '%s, ' % c.get('condition')
        result += '%s°C/%s°F,' % (c.get('temp_c'), c.get('temp_f'))
        result += '%s\n%s\n\n' % (c.get('humidity'), c.get('wind_condition'))
        f = data.get('forecast')
        for d in f:
            result += '%s: ' % (d['day_of_week'])
            result += '%s (%s°/%s°)\n' % (d['condition'], d['low'], d['high'])
        return self.strip_tags(result.decode('utf-8'))

    def render_wave(self, data, title):
        '''
        renders the result for wave responses
        '''
        i = data.get('info')
        if i:
            result = " <br/><br/>"
            result += '<b>%s</b>:<br/><br/>' % (self.htmlentities_decode(i.get('city')))#, i.get('current_date_time'))
            c = data.get('current')
            if c.get('condition'): 
                result += '%s, ' % self.htmlentities_decode(c.get('condition'))
            result += '%s°C/%s°F, %s<br/>%s<br/><br/>' % (
                    c.get('temp_c'),
                    c.get('temp_f'),
                    self.htmlentities_decode(c.get('humidity')), 
                    self.htmlentities_decode(c.get('wind_condition'))
                    )
            f = data.get('forecast')
            for day in f:
                #icon = '<img src="http://google.com%s" alt="" />' % day['icon']
                result += '%s: %s (%s°/%s°)<br/>' % (
                    self.htmlentities_decode(day['day_of_week']),
                    self.htmlentities_decode(day['condition']), 
                    day['low'], 
                    day['high']
                    )        
        else:
            result = 'Request failed'
        return result.decode('utf-8')




# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import re
import os
import new
import feedparser
from baas import plugins
from htmlentitydefs import name2codepoint as n2cp
import re


class PluginLoader(object):

    def __init__(self, config=None, format='xmpp'):
        self.config = config
        self.format = format
        self.plugins = {}
        self.help = ''
        self.commands = {}

    def load_plugins(self):
        """
            loads the plugins from the plugin dir
        """
        for file in os.listdir(plugins.__path__[0]):
            file_parts = os.path.splitext(file)
            if  file_parts[1] == '.py' and file[0:2] != '__':
                self.plugins[file_parts[0].capitalize()] = \
		getattr(__import__('baas.plugins.'+file_parts[0], globals(), \
		locals(),[file_parts[0].capitalize()]), \
		file_parts[0].capitalize())(config=self.config, format=self.format)

    def load_map(self):
        """
            combines command map
        """
        for name in self.plugins:
            cmd_map = self.plugins[name].get_map()
            if cmd_map:
                for (cmd,func) in cmd_map:
                    self.commands[cmd] = func

    def load_help(self):
        """
            combines help text, retrieved from the plugins
        """
        help_infos = {}
        help_list = []
        help_additional = []
        for name in self.plugins:
            help_info = self.plugins[name].get_help()
            if help_info:
                help_infos[name] = help_info

        for h in help_infos:
            for t in help_infos[h].get('commands'):
                help_list.append(t)
            for a in help_infos[h].get('additional'):
                help_additional.append(a)

        self.help = "\n".join(help_list)
        self.help += "\n%s" % "\n".join(help_additional)           

xml_escapes = {
    '&' : '&amp;',
    '>' : '&gt;',
    '<' : '&lt;',
    #'"' : '&#34;',
    #"'" : '&#39;'
}

class Plugin(object):

    def __init__(self, config, format='xmpp'):
        self.config = config
        self.format = format

    def get_map(self):
        return None

    def get_help(self):
        return None
    
    def strip_tags(self, value):
        """
            Return the given HTML with all tags stripped.
        """
        return re.sub(r'<[^>]*?>', '', value)        

    def xmlify(self, string):
        """
           makes a string xml valid
        """
        return re.sub(r'([&<>])', lambda m: xml_escapes[m.group()], self.strip_tags(string))

    def load_feed(self, url):
        """
            loads extenal rss/atom feed
        """
        return feedparser.parse(url)

    def substitute_entity(self, match):
        ent = match.group(3)

        if match.group(1) == "#":
            if match.group(2) == '':
                return unichr(int(ent))
            elif match.group(2) == 'x':
                return unichr(int('0x'+ent, 16))
        else:
            cp = n2cp.get(ent)

            if cp:
                return unichr(cp)
            else:
                return match.group()

    def htmlentities_decode(self, string):
        """ thanks to http://snippets.dzone.com/posts/show/4569 """
        entity_re = re.compile(r'&(#?)(x?)(\d{1,5}|\w{1,8});')
        return entity_re.subn(self.substitute_entity, string)[0]

    def render(self, data, title='', extra_format=None):
        render_func = 'render_'+self.format
        if extra_format:
            render_func += '_'+extra_format
        return  getattr(self, render_func)(data, title)

    def render_xmpp(self, data):
        pass

    def render_wave(self, data):
        pass


# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import re

def strip_tags(value):
    """
        Return the given HTML with all tags stripped.
    """
    return re.sub(r'<[^>]*?>', '', value)        

def xmlify(string):
    """
        makes a string xml valid
    """
    return re.sub(r'([&<>])', lambda m: xml_escapes[m.group()], strip_tags(string))



# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
from urllib import quote_plus
from baas.core.yqlapi import YQLApi
from baas.core.plugins import Plugin

class Yql (Plugin):

    def get_map(self):
        """
            returns the command map for the plugin
        """
        cmd_map = [('news',self.search_news), ('web', self.search_web), ('blip',self.search_blip)]
        return cmd_map

    def get_help(self):
        """
            returns the help text for the plugin
        """
        additional = '''
Some commands (news,web) can be combined with lang-codes, like #de, #en, #es etc:
news:hamburg #de
web:xmpp #de'''

        return {
            'commands': ['news:word - searches for news','web:word - websearch','blip:song - search for songs on blip.fm'],
            'additional': [additional],
        }

    def search_web(self, term):
        '''
        searches web by yahoo
        '''
        term = term.strip()
        lang = None

        if term and term.find('#')+1:
            term, lang = term.split('#',1)
            term = term.strip()

        if term == '':
            return "Please specify your search term"

        query = 'select title,url,date '
        query += 'from search.web where query="%s" ' % term

        if lang:
            query += ' AND region="%s" AND lang="%s" ' % (lang, lang)

        yql_api = YQLApi()
        response = yql_api.web(query=query)

        title = 'Searching the web for %s\n' % term
        return self.render(data=response, title=title)

    def search_news(self, term):
        '''
        searches news
        '''
        term = term.strip()
        lang = None

        if term and term.find('#')+1:
            term, lang = term.split('#',1)
            term = term.strip()

        if term == '':
            return "Please specify your search term"

        query = 'select title,url,date '
        query += 'from search.news where query="%s" ' % term

        if lang:
            query += ' AND region="%s" AND lang="%s" ' % (lang, lang)

        query += '| sort(field="age")'

        yql_api = YQLApi()
        response = yql_api.news(query=query)        
        
        title = 'Searching news for %s\n' % term
        return self.render(data=response, title=title)
        

    def search_blip(self, term):
        '''
        searches for blips on blip.fm
        '''
        term = term.strip()

        if term == '':
            return "Please specify your search term"

        yterm = 'intitle:"%s" site:blip.fm inurl:profile -intitle:"Props given" -intitle:"Favourite DJs" \
                -intitle:"Blip.fm %s"' % (term, term)

        query = 'select title,url from search.web(0,15) '
        query += "WHERE query = '%s'" % yterm

        yql_api = YQLApi()
        response = yql_api.news(query=query)

        return self.render(data=response, title='Blips for %s' % term, extra_format='blip')
        
    def render_xmpp(self, hits, title):
        '''
        renders the result for xmpp responses
        '''
        result = title+"\n"
        if hits:
            for row in hits:
                result += "(%s) %s : %s\n" % (row['date'],row['title'],row['url'])
        else:
            result += 'No hits found!'
        return self.strip_tags(result)

    def render_wave(self, hits, title):
        '''
        renders the result for wave responses
        '''
        result = " <br/><br/><b>%s</b><br/>" % self.xmlify(title)
        if hits:
            for row in hits:
                title = self.xmlify(row['title'])
                result += '<a href="%s">%s</a><br/><br/>' % (self.xmlify(row['url']), title)
        else:
            result += 'No hits found!'
        return result

    def render_xmpp_blip(self, hits, title):
        '''
        renders blip search result for xmpp responses
        '''
        result = title+"\n"
        if hits:
            for row in hits:
                result += "%s : %s\n" % (row['title'].replace('Blip.fm | ',''),row['url'])
        else:
            result +='No blips found!'
        return self.strip_tags(result)

    def render_wave_blip(self, hits, title):
        '''
        renders blip search result for wave responses
        '''
        result = " <br/><br/><b>%s</b><br/>" % self.xmlify(title)
        if hits:
            for row in hits:
                title = self.xmlify(row['title'])
                result += '<a href="%s">%s</a><br/>' % (self.xmlify(row['url']), title)
        else:
            result +='No blips found!'
        return result


from setuptools import setup, find_packages
import sys, os

version = '0.1.4.1'

setup(name='baas',
      version=version,
      description="'Buddy as a Service' is a xmpp / wavelet robot using Yahoo YQL API, Google API and other services to do searches and some other stuff (translations, weather forecast, etc) for you.",
      long_description="""\
The XMPP bot also runs on the google appengine. BaaS is easy extensible through plugins.  No API Keys required! \
See http://mborho.github.com/baas for more infos.
""",
      classifiers=[
        "Programming Language :: Python :: 2.5",
        "Topic :: Communications :: Chat", 
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Intended Audience :: Other Audience",
        "Operating System :: POSIX :: Linux",
        ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='xmpp',
      author='Martin Borho',
      author_email='martin@borho.net',
      url='http://mborho.github.com/baas',
      license='GNU General Public License (GPL)',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      #data_files=[('conf',"conf/baas.conf")],

      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'twisted',
        'feedparser',
        'chardet',
        'simplejson'
      ],
      entry_points="""
      [console_scripts]  
      baas_bot = baas.scripts.bot:main
      """,       
      )



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import urllib
import urllib2

try:
    # appengine
    from django.utils import simplejson
except:
    import simplejson

class YQLApi(object):
    def __init__(self, app_key=None, logger=None):
        self.app_key = app_key
        self.logger = logger
        
    def log(self, msg):
        if self.logger:
            self.logger.info(msg)
        
    def request(self, type_, query, **kwargs):

        self.log('Query:%s'%query)
        self.log('type_:%s'%type_)
        self.log('Other Args:%s'%kwargs)

        base_url = 'http://query.yahooapis.com/v1/public/yql?q=%s&format=json'
        base_url += '&diagnostics=false&callback='
        yql_query = urllib.quote_plus(query.encode('utf-8'))
        final_url = base_url % (yql_query)
        self.log('final_url: %s'%final_url)

        response=urllib.urlopen(final_url)
        api_data=simplejson.load(response)
        result = api_data.get('query',{}).get('results',{})

        hits = result.get('result') if result else None
        
        # handle single result
        if type(hits) == dict:
            hits = [hits]

        self.log('data:%s'% hits)        
        return hits
        
    def web(self, query, **kwargs):                    
        return self.request('web', query, **kwargs)
       
    def news(self, query, **kwargs):
       return self.request('news', query, **kwargs)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2009 Martin Borho <martin@borho.net>
# GPL - see License.txt for details
import logging
import ConfigParser
from baas.core.plugins import PluginLoader
from baas.core.helpers import *
from google.appengine.api import xmpp
from google.appengine.ext import webapp
from google.appengine.ext.webapp.util import run_wsgi_app

config = ConfigParser.ConfigParser()
config.read("../../baas.cfg")

pluginHnd = PluginLoader(config=config)
pluginHnd.load_plugins()
pluginHnd.load_map()
pluginHnd.load_help()
commands= pluginHnd.commands

class XMPPHandler(webapp.RequestHandler):

    def post(self):
        
        message = xmpp.Message(self.request.POST)
        # message.from,to,body,stanza
        text = message.body.strip()

        reply = "type 'help' for available commands"

        try:

            if text and text.find(':')+1:
                cmd,args=text.split(':',1)
                commando_func = commands.get(cmd)
                if commando_func:
                    result_msg = commando_func(args)
                    reply = result_msg
                else:
                    reply = 'Uups, commando not known\n'
            elif text and text == 'help':
                reply += "\n\n%s" % pluginHnd.help

        except Exception, e:
            logging.exception(e)
            reply = "An error occured, sorry"

        message.reply(reply)


application = webapp.WSGIApplication([('/_ah/xmpp/message/chat/', XMPPHandler)],
                                     debug=True)

def main():
  run_wsgi_app(application)

if __name__ == "__main__":
  main()
